/*
 * Open Source Physics software is free software as described near the bottom of this code file.
 *
 * For additional information and documentation on Open Source Physics please see:
 * <https://www.compadre.org/osp/>
 */

package org.opensourcephysics.controls;

/**
 * Simulations are Animations that invoke methods before starting their
 * animation thread and after stopping their animation thread. This interface is
 * a placeholder for animations that behave in a certain way.
 *
 * @author Wolfgang Christian
 * @version 1.1
 */
public interface Simulation extends Animation {
  /**
   * Tests if the simulations's animation thread is running.
   */
  public boolean isRunning();

  /**
   * Starts the simulation thread.
   */
  public void startSimulation();

  /**
   * Stops the simulation thread.
   */
  public void stopSimulation();

}

/*
 * Open Source Physics software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License (GPL) as
 * published by the Free Software Foundation; either version 2 of the License,
 * or(at your option) any later version.
 *
 * Code that uses any portion of the code in the org.opensourcephysics package
 * or any subpackage (subdirectory) of this package must must also be be
 * released under the GNU GPL license.
 *
 * This software is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston MA 02111-1307 USA or view the license online at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (c) 2019 The Open Source Physics project
 * https://www.compadre.org/osp
 */
